﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using HeuristicLab.Services.Hive.DataAccess;

namespace HeuristicLab.Services.Hive.Statistics.Controllers {
  public class ExceptionDataController : Controller {

    public JsonResult TaskExceptions(string userName, string limit, DateTime? start = null, DateTime? end = null, string jobId = null, string taskState = null) {
      using (var db = new HiveDataContext(Settings.Default.HeuristicLab_Hive_LinqConnectionString)) {
        TaskState state = GetTaskState(taskState);
        var data =
        (from errors in db.StateLogs
          join tasks in db.FactTasks
            on errors.TaskId equals tasks.TaskId
          join jobs in db.DimJobs
            on tasks.JobId equals jobs.JobId
          where jobs.UserName == userName &&
          errors.Exception != null && 
          !errors.Exception.Equals(string.Empty) &&
          (!start.HasValue || tasks.StartTime >= start) &&
          (!end.HasValue || tasks.EndTime < end) &&
          (string.IsNullOrEmpty(jobId) || tasks.JobId.ToString() == jobId) &&
          (string.IsNullOrEmpty(taskState) || tasks.TaskState == state)
          select new
          {
            TaskID = errors.TaskId,
            ErrorMessage = errors.Exception,
            StartDate = tasks.StartTime
          }).OrderByDescending(s => s.StartDate).Take(Convert.ToInt32(limit)).ToList();

        List<string> Task = new List<string>();
        List<string> Error = new List<string>();
        data.ForEach(t => Task.Add(t.TaskID.ToString()));
        data.ForEach(e => Error.Add(e.ErrorMessage));

        return Json(new KeyValuePair<List<string>, List<string>>(Task, Error), JsonRequestBehavior.AllowGet);
      }
    }

    private static TaskState GetTaskState(string taskState) {
      TaskState state = TaskState.Finished;
      switch (taskState) {
        case "Aborted":
          state = TaskState.Aborted;
          break;
        case "Calculating":
          state = TaskState.Calculating;
          break;
        case "Failed":
          state = TaskState.Failed;
          break;
        case "Finished":
          state = TaskState.Finished;
          break;
        case "Offline":
          state = TaskState.Offline;
          break;
        case "Paused":
          state = TaskState.Paused;
          break;
        case "Transferring":
          state = TaskState.Transferring;
          break;
        case "Waiting":
          state = TaskState.Waiting;
          break;
      }
      return state;
    }
  }
}
